@@ -1,11 +1,12 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
 
+import shutil
 import unittest
 import tempfile
 import pathlib
@@ -13,6 +14,8 @@ import collections
 import datetime
 import dateutil.relativedelta as relativedelta
 
+import pytest
+
 import qiime2.core.util as util
 from qiime2.core.testing.type import Foo, Bar, Baz
 
@@ -198,6 +201,31 @@ class TestMD5SumDirectory(unittest.TestCase):
                 ('foobarbaz.txt', '93b048d0202e4b06b658f3aef1e764d3')
             ]))
 
+    def test_single_file_md5sum_python(self):
+        with tempfile.TemporaryDirectory() as tempdir:
+            fp = pathlib.Path(tempdir) / 'test.txt'
+            with open(fp, 'w') as fh:
+                fh.write('contents\n')
+
+            exp = 'e66545a2155380046fce3fdbd32a6b4f'
+            obs = util.md5sum_python(fp)
+
+            self.assertEqual(exp, obs)
+
+    @pytest.mark.skipif(
+        shutil.which('md5sum') is None,
+        reason='md5sum executable is required'
+    )
+    def test_single_file_md5sum_native(self):
+        with tempfile.TemporaryDirectory() as tempdir:
+            fp = pathlib.Path(tempdir) / 'test.txt'
+            with open(fp, 'w') as fh:
+                fh.write('contents\n')
+
+            exp = 'e66545a2155380046fce3fdbd32a6b4f'
+            obs = util.md5sum_native(fp)
+            self.assertEqual(exp, obs)
+
     def test_single_file_nested(self):
         nested_dir = self.test_path / 'bar'
         nested_dir.mkdir()
