@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -44,7 +44,7 @@ def _expr(expr):
         return _build_predicate(expr.func.id, args, kwargs)
 
     if node is ast.Subscript:
-        field_expr = expr.slice.value
+        field_expr = expr.slice
 
         if type(field_expr) is ast.Tuple:
             field_expr = field_expr.elts
@@ -219,10 +219,17 @@ def ast_to_type(json_ast, scope=None):
 
         name = json_ast['name']
         if not json_ast['builtin']:
-            base_template = semantic.SemanticType(name).template
+            base_template = semantic.SemanticType(
+                name,
+                field_names=['field' + str(i) for i in range(len(fields))],
+                field_members={
+                    ('field' + str(i)):
+                    [child] for i, child in enumerate(fields)
+                },
+            ).template
         elif name == 'Visualization':
             return visualization.Visualization
-        elif name in {'List', 'Set', 'Tuple'}:
+        elif name in {'List', 'Set', 'Tuple', 'Collection'}:
             base_template = getattr(collection, name).template
         else:
             base_template = getattr(primitive, name).template
