@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -415,7 +415,7 @@ class MetadataWriter:
 
             df = md.to_dataframe(encode_missing=True)
             df.fillna('', inplace=True)
-            df = df.applymap(self._format)
+            df = df.map(self._format)
             tsv_writer.writerows(df.itertuples(index=True))
 
     def _non_default_missing(self, missing_directive):
