@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -16,6 +16,7 @@ import itertools
 import qiime2
 
 from qiime2.sdk import usage
+from qiime2.plugin.util import transform
 from qiime2.plugin.model.base import FormatBase
 
 
@@ -241,8 +242,7 @@ class TestPluginBase(unittest.TestCase):
                 shutil.copy(filepath, source_path)
         input = source_format(source_path, mode='r')
 
-        transformer = self.get_transformer(source_format, target)
-        obs = transformer(input)
+        obs = transform(input, from_type=source_format, to_type=target)
 
         if issubclass(target, FormatBase):
             self.assertIsInstance(obs, (type(pathlib.Path()), str, target))
@@ -261,3 +261,16 @@ class TestPluginBase(unittest.TestCase):
                 with self.subTest(example=name):
                     use = usage.ExecutionUsage()
                     example_f(use)
+
+
+def assert_no_nans_in_tables(fh):
+    '''
+    Checks for NaNs present in any of the tables in the indicated file then
+    resets to the head of the file.
+    '''
+    from pandas import read_html
+
+    tables = read_html(fh)
+    for df in tables:
+        assert not df.isnull().values.any()
+    fh.seek(0)
