@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -9,11 +9,15 @@
 import unittest
 import tempfile
 
+
+from qiime2.core.testing.format import SingleIntFormat
 from qiime2.core.testing.util import get_dummy_plugin
 from qiime2.plugin.testing import TestPluginBase
 
 
 class TestTesting(TestPluginBase):
+    package = 'qiime2.sdk.tests'
+
     def setUp(self):
         self.plugin = get_dummy_plugin()
 
@@ -24,6 +28,12 @@ class TestTesting(TestPluginBase):
     def tearDown(self):
         self.test_dir.cleanup()
 
+    def test_transformer_in_other_plugin(self):
+        _, obs = self.transform_format(SingleIntFormat, str,
+                                       filename='singleint.txt')
+
+        self.assertEqual('42', obs)
+
     def test_examples(self):
         self.execute_examples()
 
